package com.ruoyi.ss.store.service.impl;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ruoyi.common.domain.vo.LongDecimalVO;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.dashboard.domain.vo.BillCountVo;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.service.BonusConverter;
import com.ruoyi.ss.device.domain.DeviceCountVO;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.enums.DeviceGroupBy;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.domain.enums.DeviceStatus;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.payBill.domain.enums.PayBillBstType;
import com.ruoyi.ss.refund.domain.RefundQuery;
import com.ruoyi.ss.refund.domain.enums.RefundStatus;
import com.ruoyi.ss.refund.service.RefundService;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.domain.vo.StoreOperatorVO;
import com.ruoyi.ss.store.service.StoreAssembler;
import com.ruoyi.ss.store.service.StoreValidator;
import com.ruoyi.ss.storeStaff.domain.StoreStaffVO;
import com.ruoyi.ss.storeStaff.service.StoreStaffService;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillGroupBy;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;

/**
 * @author wjh
 *         2024/4/29
 */
@Service
public class StoreAssemblerImpl implements StoreAssembler {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private StoreValidator storeValidator;

    @Autowired
    private StoreStaffService storeStaffService;

    @Autowired
    private BonusConverter bonusConverter;

    @Autowired
    private RefundService refundService;

    /**
     * 拼接设备数量
     *
     * @param list 店铺列表
     */
    @Override
    public void assembleDeviceCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        Map<Long, Integer> countMap = deviceService
                .selectCountMapByStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setDeviceCount(count == null ? 0 : count);
        }
    }

    /**
     * 拼接营收数据
     *
     * @param list
     */
    @Override
    public void assembleRevenue(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        // 本日收入、本日订单量
        this.assembleTodayIncome(list);
        // 本月收入
        this.assembleMonthIncome(list);
        // 上月收入
        this.assembleLastMonthIncome(list);
        // 在线设备数量
        this.assembleOnlineCount(list);
        // 离线设备数量
        this.assembleOfflineCount(list);
    }

    /**
     * 拼接本月收入
     *
     * @param list
     */
    @Override
    public void assembleMonthIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        // 查询订单信息
        TransactionBillQuery query = new TransactionBillQuery();
        LocalDate now = LocalDate.now();
        query.setGroupBy(TransactionBillGroupBy.store_id.name());
        query.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        query.setStatusList(TransactionBillStatus.payedOrder());
        query.setType(TransactionBillType.RECHARGE.getType());
        query.setMonth(now.getMonthValue());
        query.setYear(now.getYear());
        Map<Long, BillCountVo> map = transactionBillService.selectCountMap(query, BillCountVo::getStoreId);

        // 查询退款信息
        RefundQuery refundQuery = new RefundQuery();
        refundQuery.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        refundQuery.setMonth(now.getMonthValue());
        refundQuery.setYear(now.getYear());
        refundQuery.setPayBillBstType(PayBillBstType.RECHARGE_ORDER.getType());
        refundQuery.setStatusList(RefundStatus.successList());
        Map<Long, BigDecimal> refundMap = refundService.selectSumOfRefundAmountGroupByStoreId(refundQuery)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        for (StoreVo store : list) {
            BillCountVo count = map.get(store.getStoreId());
            BigDecimal refundAmount = refundMap.get(store.getStoreId());
            BigDecimal monthIncome = BigDecimal.ZERO;
            if (count != null) {
                monthIncome = count.getRecharge();
            }
            if (refundAmount != null) {
                monthIncome = monthIncome.subtract(refundAmount);
            }
            store.setMonthIncome(monthIncome);
        }
    }

    /**
     * 拼接本日收入、订单量
     *
     * @param list
     */
    @Override
    public void assembleTodayIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        LocalDate now = LocalDate.now();

        // 查询订单信息
        TransactionBillQuery query = new TransactionBillQuery();
        query.setGroupBy(TransactionBillGroupBy.store_id.name());
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).filter(Objects::nonNull).distinct()
                .collect(Collectors.toList()));
        query.setStatusList(TransactionBillStatus.payedOrder());
        query.setType(TransactionBillType.RECHARGE.getType());
        query.setCreateDate(now);
        Map<Long, BillCountVo> map = transactionBillService.selectCountMap(query, BillCountVo::getStoreId);

        // 查询退款信息
        RefundQuery refundQuery = new RefundQuery();
        refundQuery.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        refundQuery.setCreateDate(now);
        refundQuery.setPayBillBstType(PayBillBstType.RECHARGE_ORDER.getType());
        refundQuery.setStatusList(RefundStatus.successList());
        Map<Long, BigDecimal> refundMap = refundService.selectSumOfRefundAmountGroupByStoreId(refundQuery)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        for (StoreVo store : list) {
            BillCountVo count = map.get(store.getStoreId());

            // 本日收入
            BigDecimal todayIncome = BigDecimal.ZERO;
            if (count != null) {
                todayIncome = count.getRecharge();
                store.setTodayOrderCount(count.getCount());
            } else {
                store.setTodayOrderCount(0);
            }

            // 退款金额
            BigDecimal refundAmount = refundMap.get(store.getStoreId());
            if (refundAmount != null) {
                todayIncome = todayIncome.subtract(refundAmount);
            }
            store.setTodayIncome(todayIncome);
        }
    }

    /**
     * 拼接上月收入
     *
     * @param list
     */
    @Override
    public void assembleLastMonthIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        // 上月
        LocalDate lastMonth = LocalDate.now().plusMonths(-1);

        // 查询订单信息
        TransactionBillQuery query = new TransactionBillQuery();
        query.setGroupBy(TransactionBillGroupBy.store_id.name());
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).filter(Objects::nonNull).distinct()
                .collect(Collectors.toList()));
        query.setStatusList(TransactionBillStatus.payedOrder());
        query.setType(TransactionBillType.RECHARGE.getType());
        query.setMonth(lastMonth.getMonthValue());
        query.setYear(lastMonth.getYear());
        Map<Long, BillCountVo> map = transactionBillService.selectCountMap(query, BillCountVo::getStoreId);

        // 查询退款信息
        RefundQuery refundQuery = new RefundQuery();
        refundQuery.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        refundQuery.setMonth(lastMonth.getMonthValue());
        refundQuery.setYear(lastMonth.getYear());
        refundQuery.setPayBillBstType(PayBillBstType.RECHARGE_ORDER.getType());
        refundQuery.setStatusList(RefundStatus.successList());
        Map<Long, BigDecimal> refundMap = refundService.selectSumOfRefundAmountGroupByStoreId(refundQuery)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        for (StoreVo store : list) {
            BillCountVo count = map.get(store.getStoreId());
            BigDecimal refundAmount = refundMap.get(store.getStoreId());
            BigDecimal lastMonthIncome = BigDecimal.ZERO;
            if (count != null) {
                lastMonthIncome = count.getRecharge();
            }
            if (refundAmount != null) {
                lastMonthIncome = lastMonthIncome.subtract(refundAmount);
            }
            store.setLastMonthIncome(lastMonthIncome);
        }
    }

    /**
     * 拼接在线数量
     *
     * @param list
     */
    @Override
    public void assembleOnlineCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        DeviceQuery query = new DeviceQuery();
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));
        query.setOnlineStatus(DeviceOnlineStatus.ONLINE.getStatus());
        query.setGroupBy(DeviceGroupBy.store_id.name());
        Map<Long, Integer> countMap = deviceService.selectCommonCountMap(query, DeviceCountVO::getStoreId);

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setOnlineCount(count == null ? 0 : count);
        }
    }

    /**
     * 拼接离线数量
     *
     * @param list
     */
    @Override
    public void assembleOfflineCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        DeviceQuery query = new DeviceQuery();
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));
        query.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());
        query.setGroupBy(DeviceGroupBy.store_id.name());
        Map<Long, Integer> countMap = deviceService.selectCommonCountMap(query, DeviceCountVO::getStoreId);

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setOfflineCount(count == null ? 0 : count);
        }
    }

    /**
     * 拼接可共享设备数量
     *
     * @param list
     */
    @Override
    public void assembleAvailableDeviceCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        DeviceQuery query = new DeviceQuery();
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));
        query.setOnlineStatus(DeviceOnlineStatus.ONLINE.getStatus());
        query.setStatus(DeviceStatus.NORMAL.getStatus());
        query.setGroupBy(DeviceGroupBy.store_id.name());
        Map<Long, Integer> countMap = deviceService.selectCommonCountMap(query, DeviceCountVO::getStoreId);

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setAvailableDeviceCount(count == null ? 0 : count);
        }
    }

    @Override
    public void assembleOperator(List<StoreVo> list, Long userId) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        for (StoreVo store : list) {
            StoreOperatorVO vo = StoreOperatorVO.defaultInstance();
            // 有操作权限，可修改、配置
            if (storeValidator.canOperaStore(store, userId)) {
                vo.setConfig(true);
                vo.setUpdate(true);
            }
            // 是商户，可删除、查看设备列表
            if (storeValidator.isMch(store, userId)) {
                vo.setDel(true);
                vo.setViewDevice(true);
                vo.setAddStaff(true);
            }
            store.setOperator(vo);
        }
    }

    @Override
    public void assembleBonusList(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        // 查询店铺员工列表
        this.assembleStaffList(list);

        for (StoreVo store : list) {
            List<Bonus> bonusList = bonusConverter.toPoListByVip(store, store.getStaffList(), true);
            store.setBonusList(bonusList);
        }
    }

    @Override
    public void assembleStaffList(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        List<StoreStaffVO> staffList = storeStaffService
                .selectListByStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));

        for (StoreVo store : list) {
            List<StoreStaffVO> storeStaffList = staffList.stream()
                    .filter(staff -> Objects.equals(staff.getStoreId(), store.getStoreId()))
                    .collect(Collectors.toList());
            store.setStaffList(storeStaffList);
        }
    }
}
